//###########################################################################
//
// FILE:    MODIFIED FOR M35 Example_2823xSci_Echoback.c
//
// TITLE:   DSP2823x Device SCI Echoback.
//
// FILE:   scia_echoback_c28.c
// TITLE:  SCI echoback Example for F28M35x.
//
//! \addtogroup control_example_list
//! <h1>SCI Echoback (scia_echoback)</h1>
//!
// To ENABLE PORT, use this in M3 core code:	SysCtlPeripheralEnable(SYSCTL_PERIPH_GPIOE); //GPIO28,29 -CSIA Rx, Tx
// To set line, use this in c28 code: 	GpioG1CtrlRegs.GPADIR.bit.GPIO29 = 1; //TX pin
// >>>>> NOTE THAT enabling port E, but setting port A <<<<<<<<
// GPIO00-GPIO07 == GPIO PortA (PA0_GPIO0  - PA7_GPIO7)  - 8 bits
// GPIO08-GPIO15 == GPIO PortB (PB0_GPIO8  - PB7_GPIO15) - 8 bits
// >>> NOTE: PORT C maps to GPIO 68-71 (4 bits)
// GPIO16-GPIO23 == GPIO PortD (PD0_GPIO16 - PD7_GPIO23) - 8 bits
// GPIO24-GPIO31 == GPIO PortE (PE0_GPIO24 - PD7_GPIO31) - 8 bits GPIO28,29 -CSIA Rx, Tx
// GPIO32-GPIO38 == GPIO PortF (PF0_GPIO32 - PF6_GPIO38) - 7 bits
// GPIO40-GPIO47 == GPIO PortG (PG0_GPIO40 - PG7_GPIO47) - 8 bits
// GPIO48-GPIO55 == GPIO PortH (PH0_GPIO48 - PH7_GPIO55) - 8 bits
// GPIO56-GPIO63 == GPIO PortJ (PJ0_GPIO56 - PJ7_GPIO63/XCLKIN) - 8 bits

// GPIO68-GPIO71 == GPIO PortC (PC4_GPIO68 - PC7_GPIO71) - 4 bits

// ex: GPIO70 == GPIO PortC, pin 6, set 0x40 = 0100 0000
// ASSUMPTIONS:
//
//    This program requires the DSP2823x header files.
//    As supplied, this project is configured for "boot to SARAM" operation.
//
//    Connect the SCI-A port to a PC via a transceiver and cable.
//    The PC application 'HyperTerminal' can be used to view the data
//    from the SCI and to send information to the SCI.  Characters received
//    by the SCI port are sent back to the host.
//
//    As supplied, this project is configured for "boot to SARAM"
//    operation.  The 2823x Boot Mode table is shown below.
//    For information on configuring the boot mode of an eZdsp,
//    please refer to the documentation included with the eZdsp,
//
//       $Boot_Table:
//
//         GPIO87   GPIO86     GPIO85   GPIO84
//          XA15     XA14       XA13     XA12
//           PU       PU         PU       PU
//        ==========================================
//            1        1          1        1    Jump to Flash
//            1        1          1        0    SCI-A boot
//            1        1          0        1    SPI-A boot
//            1        1          0        0    I2C-A boot
//            1        0          1        1    eCAN-A boot
//            1        0          1        0    McBSP-A boot
//            1        0          0        1    Jump to XINTF x16
//            1        0          0        0    Jump to XINTF x32
//            0        1          1        1    Jump to OTP
//            0        1          1        0    Parallel GPIO I/O boot
//            0        1          0        1    Parallel XINTF boot
//            0        1          0        0    Jump to SARAM	    <- "boot to SARAM"
//            0        0          1        1    Branch to check boot mode
//            0        0          1        0    Boot to flash, bypass ADC cal
//            0        0          0        1    Boot to SARAM, bypass ADC cal
//            0        0          0        0    Boot to SCI-A, bypass ADC cal
//                                              Boot_Table_End$
//
// DESCRIPTION:
//
//
//    This test recieves and echo-backs data through the SCI-A port.
//
//    1) Configure hyperterminal:
//       Use the included hyperterminal configuration file SCI_96.ht.
//       To load this configuration in hyperterminal: file->open
//       and then select the SCI_96.ht file.
//    2) Check the COM port.
//       The configuration file is currently setup for COM1.
//       If this is not correct, disconnect Call->Disconnect
//       Open the File-Properties dialog and select the correct COM port.
//    3) Connect hyperterminal Call->Call
//       and then start the 2823x SCI echoback program execution.
//    4) The program will print out a greeting and then ask you to
//       enter a character which it will echo back to hyperterminal.
//
//    As is, the program configures SCI-A for 9600 baud with
//    SYSCLKOUT = 150MHz and LSPCLK = 37.5 MHz
//    SYSCLKOUT = 100MHz and LSPCLK = 25.0 Mhz
//
//
//! \b Watch \b Variables: \n
//! - LoopCount    -  Number of characters sent
//! - ErrorCount   -  Number of errors detected
//! - SendChar     -  Character sent
//! - ReceivedChar -  Character received
//
//###########################################################################
// $TI Release: F28M35x Support Library v120 $
// $Release Date: Mon Mar  5 10:04:47 CST 2012 $
//###########################################################################

#include "DSP28x_Project.h"     // Device Headerfile and Examples Include File
#include "c28_SCIA.h"
// Prototype statements for functions found within this file.
void scia_echoback_init(void);
void scia_fifo_init(void);
void scia_xmit(int a);
void scia_msg(char *msg);

// Global counts used in this example
Uint16 LoopCount;
Uint16 ErrorCount;

// Main body of the example
void main(void) {
	Uint16 SendChar;
	Uint16 ReceivedChar;
	char *msg;
	unsigned long delay;
// Step 1. Initialize System Control:
// PLL, WatchDog, enable Peripheral Clocks
// This example function is found in the F28M35x_SysCtrl.c file.
	InitSysCtrl();

// Step 2. Initialize GPIO:
// This example function is found in the F28M35x_Gpio.c file and
// illustrates how to set the GPIO to it's default state.
//	InitGpio(); // Skipped for this example
	// Disable Protection
//=	EALLOW;
	// Set pin direction
	//GpioG1CtrlRegs.GPAxxx: GPIO 0 -31
	//GpioG1CtrlRegs.GPBxxx: GPIO 32 -63
	//GpioG1CtrlRegs.GPCxxx: GPIO 68 -71
//=	GpioG1CtrlRegs.GPCDIR.bit.GPIO70 = 1; //= 1 ==> set to output
//	GpioG1CtrlRegs.GPADIR.bit.GPIO28 = 1; //RX pin
//-!-	GpioG1CtrlRegs.GPADIR.bit.GPIO29 = 1; //TX pin
	// Disable Protection
//=	EDIS;
	// turn off LED
//=	GpioG1DataRegs.GPCDAT.bit.GPIO70 = 1;// turn off LED
//-!-	GpioG1DataRegs.GPADAT.bit.GPIO29 = 1; //TX pin

// Step 3. Clear all interrupts and initialize PIE vector table:
// Disable CPU interrupts
	DINT;

// Initialize the PIE control registers to their default state.
// The default state is all PIE interrupts disabled and flags
// are cleared.
// This function is found in the F28M35x_PieCtrl.c file.
	InitPieCtrl();

// Disable CPU interrupts and clear all CPU interrupt flags:
	IER = 0x0000;
	IFR = 0x0000;

// Initialize the PIE vector table with pointers to the shell Interrupt
// Service Routines (ISR).
// This will populate the entire table, even if the interrupt
// is not used in this example.  This is useful for debug purposes.
// The shell ISR routines are found in F28M35x_DefaultIsr.c.
// This function is found in F28M35x_PieVect.c.
	InitPieVectTable();

// Step 4. Initialize all the Device Peripherals:
// This function is found in DSP28M35x_InitPeripherals.c
//	InitPeripherals(); // Not required for this example

// Step 5. User specific code:

    LoopCount = 0;
    ErrorCount = 0;

    scia_fifo_init();	   // Initialize the SCI FIFO
    scia_echoback_init();  // Initialize SCI for echoback
// Enable global Interrupts and higher priority real-time debug events:
	EINT;  // Enable Global interrupt INTM
	ERTM;  // Enable Global realtime interrupt DBGM

    msg = "\r\n\n\nHello World!\0";
    scia_msg(msg);

    msg = "\r\nYou will enter a character, and the DSP will echo it back! \n\0";
    scia_msg(msg);
	while(1)
	{
		msg = "\r\nEnter a character: \0";
		scia_msg(msg);
/**/
//        scia_xmit(SendChar);
		// Wait for inc character
		while(SciaRegs.SCIFFRX.bit.RXFFST !=1) { } // wait for XRDY =1 for empty state

		// Get character
		ReceivedChar = SciaRegs.SCIRXBUF.all;
//        SendChar++;
//        Limit the character to 8-bits
//        SendChar &= 0x00FF;

       // Echo character back
       msg = "  You sent: \0";
       scia_msg(msg);
       scia_xmit(ReceivedChar);

       LoopCount++;

		// Turn on LED
//=		GpioG1DataRegs.GPCDAT.bit.GPIO70 = 0;
//		GpioG1DataRegs.GPADAT.bit.GPIO28 = 0; //RX pin
//-!-		GpioG1DataRegs.GPADAT.bit.GPIO29 = 0; //TX pin
		// Delay for a bit.
//		for(delay = 0; delay < 2000; delay++) {}
		// Turn off LED
//=		GpioG1DataRegs.GPCDAT.bit.GPIO70 = 1;
//		GpioG1DataRegs.GPADAT.bit.GPIO28 = 1; //RX pin
//-!-		GpioG1DataRegs.GPADAT.bit.GPIO29 = 1; //TX pin
		// Delay for a bit.
//		for(delay = 0; delay < 2000; delay++) {}
	}
}

//*****************************************************************************
// SCI initialization for this example
// Note: Clocks were previously turned on to the SCIA peripheral
// in the InitSysCtrl() function
//*****************************************************************************

// Test 1,SCIA  DLB, 8-bit word, baud rate 0x000F, default, 1 STOP bit, no parity
void scia_echoback_init()
{

	// Note: Clocks were turned on to the SCIA peripheral
    // in the InitSysCtrl() function

    // SCICCR
    //    Character length 8 bits
    //    1 stop bit
    //    Parity not enabled
    //    Idle-line mode protocol selected
    SciaRegs.SCICCR.bit.SCICHAR =(0x0007);  // 8 char bits,
    SciaRegs.SCICCR.bit.STOPBITS = 0;		// 1 stop bit,
    SciaRegs.SCICCR.bit.PARITYENA = 0;		// No parity,
    SciaRegs.SCICCR.bit.ADDRIDLE_MODE = 0;	// Idle-line mode protocol selected
	SciaRegs.SCICTL1.bit.RXENA = 1;  // enable TX, RX, internal SCICLK,
                                   // Disable RX ERR, SLEEP, TXWAKE
    SciaRegs.SCICTL1.bit.TXENA = 1;
    SciaRegs.SCICTL2.bit.TXINTENA =1;
    SciaRegs.SCICTL2.bit.RXBKINTENA =1;	//  No loopback
	#if (CPU_FRQ_150MHZ) //-!- setup gives wrong baud rate
//	      SciaRegs.SCIHBAUD    =0x0001;  // 9600 baud @LSPCLK = 37.5MHz.
//	      SciaRegs.SCILBAUD    =0x00E7;
	      SciaRegs.SCIHBAUD    =0x00;  	// 115200 baud @LSPCLK = 37.5MHz.
//	      SciaRegs.SCILBAUD    =0x27;	// it is actually 57600
	      SciaRegs.SCILBAUD    =0x13;	// it is actually 115200
	#endif
	#if (CPU_FRQ_100MHZ)//SYSCLKOUT:
      SciaRegs.SCIHBAUD    =0x0001;  // 9600 baud @LSPCLK = 20MHz.
      SciaRegs.SCILBAUD    =0x0044;
	#endif
    SciaRegs.SCICCR.bit.LOOPBKENA =0; //loopback disabled
	//change mux pins
    EALLOW;
    GpioG1CtrlRegs.GPAMUX2.all = (Uint32)(GPAMUX2_VALUE); //0x05000000 CCS reads value into ACC, 
    EDIS; //but does not write it to GPAMUX2! because it needs EALLOW. It is not mentioned in spuh22b
	SciaRegs.SCICTL1.all =0x0023;  // Release SCI from Reset
}

//*****************************************************************************
// Transmit a character from the SCI
//*****************************************************************************
void scia_xmit(int a)
{
	while (SciaRegs.SCIFFTX.bit.TXFFST != 0) {}
	SciaRegs.SCITXBUF=a;
}

//*****************************************************************************
void scia_msg(char * msg)
{
    int i;
    i = 0;
    while(msg[i] != '\0')
    {
        scia_xmit(msg[i]);
        i++;
    }
}

// Initialize the SCI FIFO
//*****************************************************************************
void scia_fifo_init()
{
	SciaRegs.SCIFFTX.all=0xE040;
	SciaRegs.SCIFFRX.all=0x2044;//f;
	SciaRegs.SCIFFCT.all=0x0;

}

